@@ -52,25 +52,28 @@ class LinuxIccToolChainFactory;
 // GccToolChain
 // --------------------------------------------------------------------------
 
-inline const QStringList languageOption(Core::Id languageId)
+inline const QStringList languageOption(Utils::Id languageId)
 {
     if (languageId == Constants::C_LANGUAGE_ID)
         return {"-x", "c"};
     return {"-x", "c++"};
 }
 
-inline const QStringList gccPredefinedMacrosOptions(Core::Id languageId)
+inline const QStringList gccPredefinedMacrosOptions(Utils::Id languageId)
 {
     return languageOption(languageId) + QStringList({"-E", "-dM"});
 }
 
 class PROJECTEXPLORER_EXPORT GccToolChain : public ToolChain
 {
+    Q_DECLARE_TR_FUNCTIONS(ProjectExplorer::GccToolChain)
+
 public:
-    GccToolChain(Core::Id typeId);
-    QString typeDisplayName() const override;
+    GccToolChain(Utils::Id typeId);
+
     Abi targetAbi() const override;
     QString originalTargetTriple() const override;
+    Utils::FilePath installDir() const override;
     QString version() const;
     Abis supportedAbis() const override;
     void setTargetAbi(const Abi &);
@@ -78,19 +81,15 @@ public:
     bool isValid() const override;
 
     Utils::LanguageExtensions languageExtensions(const QStringList &cxxflags) const override;
-    WarningFlags warningFlags(const QStringList &cflags) const override;
+    Utils::WarningFlags warningFlags(const QStringList &cflags) const override;
 
     MacroInspectionRunner createMacroInspectionRunner() const override;
-    Macros predefinedMacros(const QStringList &cxxflags) const override;
-
-    BuiltInHeaderPathsRunner createBuiltInHeaderPathsRunner() const override;
-    HeaderPaths builtInHeaderPaths(const QStringList &flags,
-                                   const Utils::FilePath &sysRootPath) const override;
+    BuiltInHeaderPathsRunner createBuiltInHeaderPathsRunner(const Utils::Environment &env) const override;
 
     void addToEnvironment(Utils::Environment &env) const override;
     Utils::FilePath makeCommand(const Utils::Environment &environment) const override;
     QStringList suggestedMkspecList() const override;
-    IOutputParser *outputParser() const override;
+    QList<Utils::OutputLineParser *> createOutputParsers() const override;
 
     QVariantMap toMap() const override;
     bool fromMap(const QVariantMap &data) override;
@@ -128,6 +127,7 @@ protected:
     void setCompilerCommand(const Utils::FilePath &path);
     void setSupportedAbis(const Abis &abis);
     void setOriginalTargetTriple(const QString &targetTriple);
+    void setInstallDir(const Utils::FilePath &installDir);
     void setMacroCache(const QStringList &allCxxflags, const Macros &macroCache) const;
     Macros macroCache(const QStringList &allCxxflags) const;
 
@@ -136,6 +136,7 @@ protected:
 
     virtual DetectedAbisResult detectSupportedAbis() const;
     virtual QString detectVersion() const;
+    virtual Utils::FilePath detectInstallDir() const;
 
     // Reinterpret options for compiler drivers inheriting from GccToolChain (e.g qcc) to apply -Wp option
     // that passes the initial options directly down to the gcc compiler
@@ -150,7 +151,7 @@ protected:
                                           const QStringList &platformCodeGenFlags,
                                           OptionsReinterpreter reinterpretOptions,
                                           HeaderPathsCache headerCache,
-                                          Core::Id languageId,
+                                          Utils::Id languageId,
                                           ExtraHeaderPathsFunction extraHeaderPathsFunction,
                                           const QStringList &flags,
                                           const QString &sysRoot,
@@ -162,25 +163,23 @@ protected:
     class WarningFlagAdder
     {
     public:
-        WarningFlagAdder(const QString &flag, WarningFlags &flags);
-        void operator ()(const char name[], WarningFlags flagsSet);
+        WarningFlagAdder(const QString &flag, Utils::WarningFlags &flags);
+        void operator ()(const char name[], Utils::WarningFlags flagsSet);
 
         bool triggered() const;
     private:
         QByteArray m_flagUtf8;
-        WarningFlags &m_flags;
+        Utils::WarningFlags &m_flags;
         bool m_doesEnable = false;
         bool m_triggered = false;
     };
 
 private:
-    explicit GccToolChain();
-
     void updateSupportedAbis() const;
     static QStringList gccPrepareArguments(const QStringList &flags,
                                            const QString &sysRoot,
                                            const QStringList &platformCodeGenFlags,
-                                           Core::Id languageId,
+                                           Utils::Id languageId,
                                            OptionsReinterpreter reinterpretOptions);
 
 protected:
@@ -197,6 +196,7 @@ private:
     mutable QString m_originalTargetTriple;
     mutable HeaderPaths m_headerPaths;
     mutable QString m_version;
+    mutable Utils::FilePath m_installDir;
 
     friend class Internal::GccToolChainConfigWidget;
     friend class Internal::GccToolChainFactory;
@@ -209,18 +209,19 @@ private:
 
 class PROJECTEXPLORER_EXPORT ClangToolChain : public GccToolChain
 {
+    Q_DECLARE_TR_FUNCTIONS(ProjectExplorer::ClangToolChain)
+
 public:
     ClangToolChain();
-    explicit ClangToolChain(Core::Id typeId);
+    explicit ClangToolChain(Utils::Id typeId);
     ~ClangToolChain() override;
 
-    QString typeDisplayName() const override;
     Utils::FilePath makeCommand(const Utils::Environment &environment) const override;
 
     Utils::LanguageExtensions languageExtensions(const QStringList &cxxflags) const override;
-    WarningFlags warningFlags(const QStringList &cflags) const override;
+    Utils::WarningFlags warningFlags(const QStringList &cflags) const override;
 
-    IOutputParser *outputParser() const override;
+    QList<Utils::OutputLineParser *> createOutputParsers() const override;
 
     QStringList suggestedMkspecList() const override;
     void addToEnvironment(Utils::Environment &env) const override;
@@ -228,7 +229,8 @@ public:
     QString originalTargetTriple() const override;
     QString sysRoot() const override;
 
-    BuiltInHeaderPathsRunner createBuiltInHeaderPathsRunner() const override;
+    BuiltInHeaderPathsRunner createBuiltInHeaderPathsRunner(
+            const Utils::Environment &env) const override;
 
     std::unique_ptr<ToolChainConfigWidget> createConfigurationWidget() override;
 
@@ -255,8 +257,9 @@ private:
 
 class PROJECTEXPLORER_EXPORT MingwToolChain : public GccToolChain
 {
+    Q_DECLARE_TR_FUNCTIONS(ProjectExplorer::MingwToolChain)
+
 public:
-    QString typeDisplayName() const override;
     Utils::FilePath makeCommand(const Utils::Environment &environment) const override;
 
     QStringList suggestedMkspecList() const override;
@@ -274,11 +277,11 @@ private:
 
 class PROJECTEXPLORER_EXPORT LinuxIccToolChain : public GccToolChain
 {
-public:
-    QString typeDisplayName() const override;
+    Q_DECLARE_TR_FUNCTIONS(ProjectExplorer::LinuxIccToolChain)
 
+public:
     Utils::LanguageExtensions languageExtensions(const QStringList &cxxflags) const override;
-    IOutputParser *outputParser() const override;
+    QList<Utils::OutputLineParser *> createOutputParsers() const override;
 
     QStringList suggestedMkspecList() const override;
 
