@@ -36,22 +36,22 @@
 #include <QPointer>
 #include <qtimer.h>
 
-#include <math.h>
+#include <cmath>
 
 namespace TextEditor {
 
 class SyntaxHighlighterPrivate
 {
-    SyntaxHighlighter *q_ptr;
+    SyntaxHighlighter *q_ptr = nullptr;
     Q_DECLARE_PUBLIC(SyntaxHighlighter)
 public:
-    inline SyntaxHighlighterPrivate()
-        : q_ptr(0), rehighlightPending(false), inReformatBlocks(false)
-    {}
+    SyntaxHighlighterPrivate()
+    {
+        updateFormats(TextEditorSettings::fontSettings());
+    }
 
     QPointer<QTextDocument> doc;
 
-    void _q_reformatBlocks(int from, int charsRemoved, int charsAdded);
     void reformatBlocks(int from, int charsRemoved, int charsAdded);
     void reformatBlock(const QTextBlock &block, int from, int charsRemoved, int charsAdded);
 
@@ -63,70 +63,66 @@ public:
         inReformatBlocks = false;
     }
 
-    inline void _q_delayedRehighlight() {
-        if (!rehighlightPending)
-            return;
-        rehighlightPending = false;
-        q_func()->rehighlight();
-    }
-
     void applyFormatChanges(int from, int charsRemoved, int charsAdded);
-    void updateFormatsForCategories(const FontSettings &fontSettings);
+    void updateFormats(const FontSettings &fontSettings);
 
+    FontSettings fontSettings;
     QVector<QTextCharFormat> formatChanges;
     QTextBlock currentBlock;
-    bool rehighlightPending;
-    bool inReformatBlocks;
+    bool rehighlightPending = false;
+    bool inReformatBlocks = false;
     TextDocumentLayout::FoldValidator foldValidator;
     QVector<QTextCharFormat> formats;
-    QVector<TextStyle> formatCategories;
+    QVector<std::pair<int,TextStyle>> formatCategories;
+    QTextCharFormat whitespaceFormat;
+    bool noAutomaticHighlighting = false;
 };
 
-static bool adjustRange(QTextLayout::FormatRange &range, int from, int charsRemoved, int charsAdded) {
-
+static bool adjustRange(QTextLayout::FormatRange &range, int from, int charsDelta)
+{
     if (range.start >= from) {
-        range.start += charsAdded - charsRemoved;
+        range.start += charsDelta;
         return true;
     } else if (range.start + range.length > from) {
-        range.length += charsAdded - charsRemoved;
+        range.length += charsDelta;
         return true;
     }
     return false;
 }
 
+void SyntaxHighlighter::delayedRehighlight()
+{
+    Q_D(SyntaxHighlighter);
+    if (!d->rehighlightPending)
+        return;
+    d->rehighlightPending = false;
+    rehighlight();
+}
+
 void SyntaxHighlighterPrivate::applyFormatChanges(int from, int charsRemoved, int charsAdded)
 {
     bool formatsChanged = false;
 
     QTextLayout *layout = currentBlock.layout();
 
-    QList<QTextLayout::FormatRange> ranges = layout->additionalFormats();
+    QVector<QTextLayout::FormatRange> ranges;
+    QVector<QTextLayout::FormatRange> oldRanges;
+    std::tie(ranges, oldRanges)
+        = Utils::partition(layout->formats(), [](const QTextLayout::FormatRange &range) {
+              return range.format.property(QTextFormat::UserProperty).toBool();
+          });
 
-    bool doAdjustRange = currentBlock.contains(from);
-
-    QList<QTextLayout::FormatRange> old_ranges;
-
-    if (!ranges.isEmpty()) {
-        QList<QTextLayout::FormatRange>::Iterator it = ranges.begin();
-        while (it != ranges.end()) {
-            if (it->format.property(QTextFormat::UserProperty).toBool()) {
-                if (doAdjustRange)
-                    formatsChanged = adjustRange(*it, from - currentBlock.position(), charsRemoved, charsAdded)
-                            || formatsChanged;
-                ++it;
-            } else {
-                old_ranges.append(*it);
-                it = ranges.erase(it);
-            }
-        }
+    if (currentBlock.contains(from)) {
+        const int charsDelta = charsAdded - charsRemoved;
+        for (QTextLayout::FormatRange &range : ranges)
+            formatsChanged |= adjustRange(range, from - currentBlock.position(), charsDelta);
     }
 
     QTextCharFormat emptyFormat;
 
     QTextLayout::FormatRange r;
-    r.start = -1;
 
-    QList<QTextLayout::FormatRange> new_ranges;
+    QVector<QTextLayout::FormatRange> newRanges;
     int i = 0;
     while (i < formatChanges.count()) {
 
@@ -142,40 +138,31 @@ void SyntaxHighlighterPrivate::applyFormatChanges(int from, int charsRemoved, in
         while (i < formatChanges.count() && formatChanges.at(i) == r.format)
             ++i;
 
-        if (i >= formatChanges.count())
-            break;
-
         r.length = i - r.start;
 
-        new_ranges << r;
-        r.start = -1;
+        newRanges << r;
     }
 
-    if (r.start != -1) {
-        r.length = formatChanges.count() - r.start;
+    formatsChanged = formatsChanged || (newRanges.size() != oldRanges.size());
 
-        new_ranges << r;
-    }
-
-    formatsChanged = formatsChanged || (new_ranges.size() != old_ranges.size());
-
-    for (int i = 0; !formatsChanged && i < new_ranges.size(); ++i) {
-        const QTextLayout::FormatRange &o = old_ranges.at(i);
-        const QTextLayout::FormatRange &n = new_ranges.at(i);
+    for (int i = 0; !formatsChanged && i < newRanges.size(); ++i) {
+        const QTextLayout::FormatRange &o = oldRanges.at(i);
+        const QTextLayout::FormatRange &n = newRanges.at(i);
         formatsChanged = (o.start != n.start || o.length != n.length || o.format != n.format);
     }
 
     if (formatsChanged) {
-        ranges.append(new_ranges);
-        layout->setAdditionalFormats(ranges);
+        ranges.append(newRanges);
+        layout->setFormats(ranges);
         doc->markContentsDirty(currentBlock.position(), currentBlock.length());
     }
 }
 
-void SyntaxHighlighterPrivate::_q_reformatBlocks(int from, int charsRemoved, int charsAdded)
+void SyntaxHighlighter::reformatBlocks(int from, int charsRemoved, int charsAdded)
 {
-    if (!inReformatBlocks)
-        reformatBlocks(from, charsRemoved, charsAdded);
+    Q_D(SyntaxHighlighter);
+    if (!d->inReformatBlocks)
+        d->reformatBlocks(from, charsRemoved, charsAdded);
 }
 
 void SyntaxHighlighterPrivate::reformatBlocks(int from, int charsRemoved, int charsAdded)
@@ -238,10 +225,10 @@ void SyntaxHighlighterPrivate::reformatBlock(const QTextBlock &block, int from,
     The SyntaxHighlighter class is a copied and forked version of the QSyntaxHighlighter. There are
     a couple of binary incompatible changes that prevent doing this directly in Qt.
 
-    The main difference from the QSyntaxHighlighter is the addition of setExtraAdditionalFormats.
-    This method prevents redoing syntax highlighting when setting the additionalFormats on the
+    The main difference from the QSyntaxHighlighter is the addition of setExtraFormats.
+    This method prevents redoing syntax highlighting when setting the formats on the
     layout and subsequently marking the document contents dirty. It thus prevents the redoing of the
-    semantic highlighting, which sets extra additionalFormats, and so on.
+    semantic highlighting, which sets extra formats, and so on.
 
     Another way to implement the semantic highlighting is to use ExtraSelections on
     Q(Plain)TextEdit. The drawback of QTextEdit::setExtraSelections is that ExtraSelection uses a
@@ -255,7 +242,7 @@ void SyntaxHighlighterPrivate::reformatBlock(const QTextBlock &block, int from,
     document. This means that every editor needs a highlighter, instead of every document. This
     could become expensive when multiple editors with the same document are opened.
 
-    So, we use AdditionalFormats, because all those highlights should get removed or redone soon
+    So, we use formats, because all those highlights should get removed or redone soon
     after the change happens.
 */
 
@@ -299,7 +286,7 @@ SyntaxHighlighter::SyntaxHighlighter(QTextEdit *parent)
 */
 SyntaxHighlighter::~SyntaxHighlighter()
 {
-    setDocument(0);
+    setDocument(nullptr);
 }
 
 /*!
@@ -310,21 +297,21 @@ void SyntaxHighlighter::setDocument(QTextDocument *doc)
 {
     Q_D(SyntaxHighlighter);
     if (d->doc) {
-        disconnect(d->doc, SIGNAL(contentsChange(int,int,int)),
-                   this, SLOT(_q_reformatBlocks(int,int,int)));
+        disconnect(d->doc, &QTextDocument::contentsChange, this, &SyntaxHighlighter::reformatBlocks);
 
         QTextCursor cursor(d->doc);
         cursor.beginEditBlock();
         for (QTextBlock blk = d->doc->begin(); blk.isValid(); blk = blk.next())
-            blk.layout()->clearAdditionalFormats();
+            blk.layout()->clearFormats();
         cursor.endEditBlock();
     }
     d->doc = doc;
     if (d->doc) {
-        connect(d->doc, SIGNAL(contentsChange(int,int,int)),
-                this, SLOT(_q_reformatBlocks(int,int,int)));
-        d->rehighlightPending = true;
-        QTimer::singleShot(0, this, SLOT(_q_delayedRehighlight()));
+        if (!d->noAutomaticHighlighting) {
+            connect(d->doc, &QTextDocument::contentsChange, this, &SyntaxHighlighter::reformatBlocks);
+            d->rehighlightPending = true;
+            QTimer::singleShot(0, this, &SyntaxHighlighter::delayedRehighlight);
+        }
         d->foldValidator.setup(qobject_cast<TextDocumentLayout *>(doc->documentLayout()));
     }
 }
@@ -484,22 +471,56 @@ void SyntaxHighlighter::setFormat(int start, int count, const QFont &font)
     setFormat(start, count, format);
 }
 
-void SyntaxHighlighter::applyFormatToSpaces(const QString &text, const QTextCharFormat &format)
+void SyntaxHighlighter::formatSpaces(const QString &text, int start, int count)
 {
-    int offset = 0;
-    const int length = text.length();
-    while (offset < length) {
+    Q_D(const SyntaxHighlighter);
+    int offset = start;
+    const int end = std::min(start + count, int(text.length()));
+    while (offset < end) {
         if (text.at(offset).isSpace()) {
             int start = offset++;
-            while (offset < length && text.at(offset).isSpace())
+            while (offset < end && text.at(offset).isSpace())
                 ++offset;
-            setFormat(start, offset - start, format);
+            setFormat(start, offset - start, d->whitespaceFormat);
         } else {
             ++offset;
         }
     }
 }
 
+/*!
+    The specified \a format is applied to all non-whitespace characters in the current text block
+    with \a text, from the \a start position for a length of \a count characters.
+    Whitespace characters are formatted with the visual whitespace format, merged with the
+    non-whitespace format.
+
+    \sa setFormat()
+*/
+void SyntaxHighlighter::setFormatWithSpaces(const QString &text, int start, int count,
+                                            const QTextCharFormat &format)
+{
+    Q_D(const SyntaxHighlighter);
+    QTextCharFormat visualSpaceFormat = d->whitespaceFormat;
+    visualSpaceFormat.setBackground(format.background());
+
+    const int end = std::min(start + count, int(text.length()));
+    int index = start;
+
+    while (index != end) {
+        const bool isSpace = text.at(index).isSpace();
+        const int start = index;
+
+        do { ++index; }
+        while (index != end && text.at(index).isSpace() == isSpace);
+
+        const int tokenLength = index - start;
+        if (isSpace)
+            setFormat(start, tokenLength, visualSpaceFormat);
+        else if (format.isValid())
+            setFormat(start, tokenLength, format);
+    }
+}
+
 /*!
     Returns the format at \a position inside the syntax highlighter's
     current text block.
@@ -612,7 +633,7 @@ QTextBlockUserData *SyntaxHighlighter::currentBlockUserData() const
 {
     Q_D(const SyntaxHighlighter);
     if (!d->currentBlock.isValid())
-        return 0;
+        return nullptr;
 
     return d->currentBlock.userData();
 }
@@ -633,59 +654,33 @@ static bool byStartOfRange(const QTextLayout::FormatRange &range, const QTextLay
     return range.start < other.start;
 }
 
-// The formats is passed in by reference in order to prevent unnecessary copying of its items.
+// The formats is passed in by rvalue reference in order to prevent unnecessary copying of its items.
 // After this function returns, the list is modified, and should be considered invalidated!
-void SyntaxHighlighter::setExtraAdditionalFormats(const QTextBlock& block,
-                                                  QList<QTextLayout::FormatRange> &formats)
+void SyntaxHighlighter::setExtraFormats(const QTextBlock &block,
+                                        QVector<QTextLayout::FormatRange> &&formats)
 {
-//    qDebug() << "setAdditionalFormats() on block" << block.blockNumber();
-//    qDebug() << "   is valid:" << (block.isValid() ? "Yes" : "No");
-//    qDebug() << "   has layout:" << (block.layout() ? "Yes" : "No");
-//    if (block.layout()) qDebug() << "   has text:" << (block.text().isEmpty() ? "No" : "Yes");
-
-//    for (int i = 0; i < overrides.count(); ++i)
-//        qDebug() << "   from " << overrides.at(i).start << "length"
-//                 << overrides.at(i).length
-//                 << "color:" << overrides.at(i).format.foreground().color();
     Q_D(SyntaxHighlighter);
 
     const int blockLength = block.length();
-    if (block.layout() == 0 || blockLength == 0)
+    if (block.layout() == nullptr || blockLength == 0)
         return;
 
     Utils::sort(formats, byStartOfRange);
 
-    const QList<QTextLayout::FormatRange> all = block.layout()->additionalFormats();
-    QList<QTextLayout::FormatRange> previousSemanticFormats;
-    QList<QTextLayout::FormatRange> formatsToApply;
-    previousSemanticFormats.reserve(all.size());
-    formatsToApply.reserve(all.size() + formats.size());
+    const QVector<QTextLayout::FormatRange> all = block.layout()->formats();
+    QVector<QTextLayout::FormatRange> previousSemanticFormats;
+    QVector<QTextLayout::FormatRange> formatsToApply;
+    std::tie(previousSemanticFormats, formatsToApply)
+        = Utils::partition(all, [](const QTextLayout::FormatRange &r) {
+              return r.format.hasProperty(QTextFormat::UserProperty);
+          });
 
-    for (int i = 0, ei = formats.size(); i < ei; ++i)
-        formats[i].format.setProperty(QTextFormat::UserProperty, true);
-
-    foreach (const QTextLayout::FormatRange &r, all) {
-        if (r.format.hasProperty(QTextFormat::UserProperty))
-            previousSemanticFormats.append(r);
-        else
-            formatsToApply.append(r);
-    }
+    for (auto &format : formats)
+        format.format.setProperty(QTextFormat::UserProperty, true);
 
     if (formats.size() == previousSemanticFormats.size()) {
         Utils::sort(previousSemanticFormats, byStartOfRange);
-
-        int index = 0;
-        for (; index != formats.size(); ++index) {
-            const QTextLayout::FormatRange &range = formats.at(index);
-            const QTextLayout::FormatRange &previousRange = previousSemanticFormats.at(index);
-
-            if (range.start != previousRange.start ||
-                    range.length != previousRange.length ||
-                    range.format != previousRange.format)
-                break;
-        }
-
-        if (index == formats.size())
+        if (formats == previousSemanticFormats)
             return;
     }
 
@@ -693,11 +688,40 @@ void SyntaxHighlighter::setExtraAdditionalFormats(const QTextBlock& block,
 
     bool wasInReformatBlocks = d->inReformatBlocks;
     d->inReformatBlocks = true;
-    block.layout()->setAdditionalFormats(formatsToApply);
+    block.layout()->setFormats(formatsToApply);
     document()->markContentsDirty(block.position(), blockLength - 1);
     d->inReformatBlocks = wasInReformatBlocks;
 }
 
+void SyntaxHighlighter::clearExtraFormats(const QTextBlock &block)
+{
+    Q_D(SyntaxHighlighter);
+
+    const int blockLength = block.length();
+    if (block.layout() == nullptr || blockLength == 0)
+        return;
+
+    const QVector<QTextLayout::FormatRange> formatsToApply
+        = Utils::filtered(block.layout()->formats(), [](const QTextLayout::FormatRange &r) {
+              return !r.format.hasProperty(QTextFormat::UserProperty);
+          });
+
+    bool wasInReformatBlocks = d->inReformatBlocks;
+    d->inReformatBlocks = true;
+    block.layout()->setFormats(formatsToApply);
+    document()->markContentsDirty(block.position(), blockLength - 1);
+    d->inReformatBlocks = wasInReformatBlocks;
+}
+
+void SyntaxHighlighter::clearAllExtraFormats()
+{
+    QTextBlock b = document()->firstBlock();
+    while (b.isValid()) {
+        clearExtraFormats(b);
+        b = b.next();
+    }
+}
+
 /* Generate at least n different colors for highlighting, excluding background
  * color. */
 
@@ -707,7 +731,7 @@ QList<QColor> SyntaxHighlighter::generateColors(int n, const QColor &background)
     // Assign a color gradient. Generate a sufficient number of colors
     // by using ceil and looping from 0..step.
     const double oneThird = 1.0 / 3.0;
-    const int step = qRound(ceil(pow(double(n), oneThird)));
+    const int step = qRound(std::ceil(std::pow(double(n), oneThird)));
     result.reserve(step * step * step);
     const int factor = 255 / step;
     const int half = factor / 2;
@@ -735,14 +759,68 @@ QList<QColor> SyntaxHighlighter::generateColors(int n, const QColor &background)
 void SyntaxHighlighter::setFontSettings(const FontSettings &fontSettings)
 {
     Q_D(SyntaxHighlighter);
-    d->updateFormatsForCategories(fontSettings);
+    d->updateFormats(fontSettings);
 }
 
-void SyntaxHighlighter::setTextFormatCategories(const QVector<TextStyle> &categories)
+FontSettings SyntaxHighlighter::fontSettings() const
+{
+    Q_D(const SyntaxHighlighter);
+    return d->fontSettings;
+}
+/*!
+    The syntax highlighter is not anymore reacting to the text document if \a noAutmatic is
+    \c true.
+*/
+void SyntaxHighlighter::setNoAutomaticHighlighting(bool noAutomatic)
+{
+    Q_D(SyntaxHighlighter);
+    d->noAutomaticHighlighting = noAutomatic;
+}
+
+/*!
+    Creates text format categories for the text styles themselves, so the highlighter can
+    use \c{formatForCategory(C_COMMENT)} and similar, and avoid creating its own format enum.
+    \sa setTextFormatCategories()
+*/
+void SyntaxHighlighter::setDefaultTextFormatCategories()
+{
+    // map all text styles to themselves
+    setTextFormatCategories(C_LAST_STYLE_SENTINEL, [](int i) { return TextStyle(i); });
+}
+
+/*!
+    Uses the \a formatMapping function to create a mapping from the custom formats (the ints)
+    to text styles. The \a formatMapping must handle all values from 0 to \a count.
+
+    \sa setDefaultTextFormatCategories()
+    \sa setTextFormatCategories()
+*/
+void SyntaxHighlighter::setTextFormatCategories(int count,
+                                                std::function<TextStyle(int)> formatMapping)
+{
+    QVector<std::pair<int, TextStyle>> categories;
+    categories.reserve(count);
+    for (int i = 0; i < count; ++i)
+        categories.append({i, formatMapping(i)});
+    setTextFormatCategories(categories);
+}
+
+/*!
+    Creates a mapping between custom format enum values (the int values in the pairs) to
+    text styles. Afterwards \c{formatForCategory(MyCustomFormatEnumValue)} can be used to
+    efficiently retrieve the text style for a value.
+
+    Note that this creates a vector with a size of the maximum int value in \a categories.
+
+    \sa setDefaultTextFormatCategories()
+*/
+void SyntaxHighlighter::setTextFormatCategories(const QVector<std::pair<int, TextStyle>> &categories)
 {
     Q_D(SyntaxHighlighter);
     d->formatCategories = categories;
-    d->updateFormatsForCategories(TextEditorSettings::fontSettings());
+    const int maxCategory = Utils::maxElementOr(categories, {-1, C_TEXT}).first;
+    d->formats = QVector<QTextCharFormat>(maxCategory + 1);
+    d->updateFormats(TextEditorSettings::fontSettings());
 }
 
 QTextCharFormat SyntaxHighlighter::formatForCategory(int category) const
@@ -753,9 +831,21 @@ QTextCharFormat SyntaxHighlighter::formatForCategory(int category) const
     return d->formats.at(category);
 }
 
-void SyntaxHighlighterPrivate::updateFormatsForCategories(const FontSettings &fontSettings)
+void SyntaxHighlighter::highlightBlock(const QString &text)
 {
-    formats = fontSettings.toTextCharFormats(formatCategories);
+    formatSpaces(text);
+}
+
+void SyntaxHighlighterPrivate::updateFormats(const FontSettings &fontSettings)
+{
+    this->fontSettings = fontSettings;
+    // C_TEXT is handled by text editor's foreground and background color,
+    // so use empty format for that
+    for (const auto &pair : qAsConst(formatCategories)) {
+        formats[pair.first] = pair.second == C_TEXT ? QTextCharFormat()
+                                                    : fontSettings.toTextCharFormat(pair.second);
+    }
+    whitespaceFormat = fontSettings.toTextCharFormat(C_VISUAL_WHITESPACE);
 }
 
 } // namespace TextEditor
